<?php

/**
 * @file
 * Menu builder functions for Term Merge Manager.
 */

/**
 * Form builder function for module settings - manage term actions.
 */
function term_merge_manager_settings($form, $form_state) {
  $form['actions'] = term_merge_manager_actions();

  return $form;
}

/**
 * Build Admin Table of all single actions.
 */
function term_merge_manager_actions() {
  // Build the sortable table header.
  $header = array(
    'tmmiid' => array('data' => t('TMMID'), 'field' => 'tmmi.tmmiid'),
    'vid' => array('data' => t('Vocabulary'), 'field' => 'tmmi.vid'),
    'from' => array('data' => t('From'), 'field' => 'tmmi.name'),
    'into' => array('data' => t('Into'), 'field' => 'tmm.tid'),
  );

  $header['operations'] = array('data' => t('Operations'));

  $query = db_select('term_merge_manager_item', 'tmmi')->extend('PagerDefault')->extend('TableSort');
  $query->addTag('term_merge_manager_settings');

  $tmmitems = $query
    ->fields('tmmi', array('name', 'vid', 'tmmid', 'tmmiid'))
    ->limit(50)
    ->orderByHeader($header)
    ->execute()
    ->fetchAll();

  // Prepare the list of actions.
  $destination = drupal_get_destination();
  $items = array();
  $vocabs = array();
  $tmmids = array();
  foreach ($tmmitems as $tmmitem) {

    $tmmid = $tmmitem->tmmid;
    $tmmiid = $tmmitem->tmmiid;

    if (!isset($tmmids[$tmmid])) {
      $tmmids[$tmmid] = term_merge_manager_load_tmmid($tmmid);
    }

    $tid = $tmmids[$tmmid][0]->tid;

    if (!isset($vocabs[$tmmitem->vid])) {
      $vocabs[$tmmitem->vid] = taxonomy_vocabulary_load($tmmitem->vid);
    }
    $vocablabel = $vocabs[$tmmitem->vid]->name . ' (VID: ' . $tmmitem->vid . ')';

    $toterm = taxonomy_term_load($tid);

    $items[$tmmiid] = array(
      'tmmiid' => check_plain($tmmiid),
      'vid' => check_plain($vocablabel),
      'from' => check_plain($tmmitem->name),
      'into' => check_plain($toterm->name . ' (TID: ' . $tid . ')'),
    );

    // Build a list of all the accessible operations for the current action.
    $operations = array();
    // TODO: edit
    // $operations['edit'] = array(
    // 'title' => t('edit'),
    // 'href' => 'node/' . $node->nid . '/edit',
    // 'query' => $destination,
    // );
    $operations['delete'] = array(
      'title' => t('delete'),
      'href' => 'admin/config/system/term_merge_manager/' . $tmmiid . '/delete',
      'query' => $destination,
    );

    $items[$tmmiid]['operations'] = array();
    if (count($operations) > 1) {
      // Render an unordered list of operations links.
      $items[$tmmiid]['operations'] = array(
        'data' => array(
          '#theme' => 'links__node_operations',
          '#links' => $operations,
          '#attributes' => array('class' => array('links', 'inline')),
        ),
      );
    }
    elseif (!empty($operations)) {
      // Render the first and only operation as a link.
      $link = reset($operations);
      $items[$tmmiid]['operations'] = array(
        'data' => array(
          '#type' => 'link',
          '#title' => $link['title'],
          '#href' => $link['href'],
          '#options' => array('query' => $link['query']),
        ),
      );
    }
  }

  // TODO: checkboxes for bulk action
  // $form['actions'] = array(
  // '#type' => 'tableselect',
  // '#header' => $header,
  // '#options' => $items,
  // '#empty' => t('No content available.'),
  // );
  $form['actions'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $items,
    '#empty' => t('No content available.'),
  );

  $form['pager'] = array('#markup' => theme('pager'));
  return $form;
}

/**
 * Load term merge manager from database.
 */
function term_merge_manager_load_tmmid($tmmid) {
  $query = db_select('term_merge_manager', 'tmm');
  return $query->fields('tmm', array('tmmid', 'vid', 'tid'))
    ->condition('tmmid', $tmmid, '=')
    ->execute()
    ->fetchAll();
}

/**
 * Form constructor for the term merge action deletion confirmation form.
 *
 * @see term_merge_manager_action_delete_confirm_submit()
 */
function term_merge_manager_action_delete_confirm($form, &$form_state, $tmmiid) {
  $form['#tmmiid'] = $tmmiid;
  // Always provide entity id in the same form key as in the entity edit form.
  $form['tmmiid'] = array('#type' => 'value', '#value' => $tmmiid);
  return confirm_form($form,
      t('Are you sure you want to delete TMMIID: %tmmiid?', array('%tmmiid' => $tmmiid)),
      'admin/config/system/term_merge_manager',
      t('This action cannot be undone.'),
      t('Delete'),
      t('Cancel')
      );
}

/**
 * Executes term merge action deletion.
 *
 * @see term_merge_manager_action_delete_confirm()
 */
function term_merge_manager_action_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    $tmmiid = $form_state['values']['tmmiid'];
    term_merge_manager_delete_tmmiid($tmmiid);
    drupal_set_message(t('TMMIID: %tmmiid has been deleted.', array('%tmmiid' => $tmmiid)));
  }

  $form_state['redirect'] = 'admin/config/system/term_merge_manager';
}

/**
 * Executes term merge action deletion.
 *
 * @see term_merge_manager_action_delete_confirm()
 */
function term_merge_manager_delete_tmmiid($tmmiid) {
  db_delete('term_merge_manager_item')
    ->condition('tmmiid', $tmmiid)
    ->execute();
}
